unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, RVScroll, RichView, RVStyle;

type
  TForm1 = class(TForm)
    rvs: TRVStyle;
    rv: TRichView;
    Edit: TEdit;
    btnEnter: TButton;
    btnBold: TButton;
    btnItalic: TButton;
    btnUnderline: TButton;
    Button1: TButton;
    Button2: TButton;
    ColorDialog1: TColorDialog;
    Label1: TLabel;
    procedure btnBoldClick(Sender: TObject);
    procedure btnItalicClick(Sender: TObject);
    procedure btnUnderlineClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure btnEnterClick(Sender: TObject);
    procedure EditKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    function GetStyleNo(CommandChar: Char; Color: TColor; StyleNo: Integer): Integer;
    procedure ParseString(s: String);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}
{------------------------------------------------------------------------------}
// TColor to RGB string
function GetColorStr(Color: TColor): String;
begin
  Result := IntToHex(ColorToRGB(Color),6);
  Result := Copy(Result,5,2)+Copy(Result,3,2)+Copy(Result,1,2);
end;
// RGB string to TColor
function GetStrColor(ColorValue: String): TColor;
var i,v: Integer;
begin
  ColorValue := Copy(ColorValue,5,2)+Copy(ColorValue,3,2)+Copy(ColorValue,1,2);
  Result := 0;
  for i := 1 to 6 do begin
    if ColorValue[i] in ['A'..'F'] then
      v := ord(ColorValue[i])-ord('A')+10
    else if ColorValue[i] in ['a'..'f'] then
      v := ord(ColorValue[i])-ord('a')+10
    else
      v := ord(ColorValue[i])-ord('0');
    Result := Result*16+v;
  end;
end;
{------------------------------------------------------------------------------}
// Insert command: bold on/off
procedure TForm1.btnBoldClick(Sender: TObject);
begin
  Edit.SelText := '\b';
  Edit.SetFocus;
end;
{------------------------------------------------------------------------------}
// Insert command: italic on/off
procedure TForm1.btnItalicClick(Sender: TObject);
begin
  Edit.SelText := '\i';
  Edit.SetFocus;
end;
{------------------------------------------------------------------------------}
// Insert command: underline on/off
procedure TForm1.btnUnderlineClick(Sender: TObject);
begin
  Edit.SelText := '\u';
  Edit.SetFocus;
end;
{------------------------------------------------------------------------------}
// Insert command: text color
procedure TForm1.Button1Click(Sender: TObject);
begin
  ColorDialog1.Color := clBlack;
  if ColorDialog1.Execute then
    Edit.SelText := '\c'+GetColorStr(ColorDialog1.Color);
  Edit.SetFocus;
end;
{------------------------------------------------------------------------------}
// Insert command: text background color
procedure TForm1.Button2Click(Sender: TObject);
begin
  ColorDialog1.Color := clWhite;
  if ColorDialog1.Execute then
    Edit.SelText := '\k'+GetColorStr(ColorDialog1.Color);
  Edit.SetFocus;
end;
{------------------------------------------------------------------------------}
// Adding string in RichView
procedure TForm1.btnEnterClick(Sender: TObject);
begin
  ParseString(Edit.Text);
  rv.FormatTail;
  rv.Refresh;
//  Edit.Text := '';
end;

procedure TForm1.EditKeyPress(Sender: TObject; var Key: Char);
begin
  if Key=#13 then begin
    Key := #0;
    btnEnterClick(Sender);
  end;
end;
{------------------------------------------------------------------------------}
// Returning StyleNo basing on command
// 'b' - bold on/off, Color is ignored
// 'i' - italic on/off, Color is ignored
// 'u' - underline on/off, Color is ignored
// 'c' - set text Color
// 'k' - set background Color
function TForm1.GetStyleNo(CommandChar: Char; Color: TColor;
  StyleNo: Integer): Integer;
var fi: TFontInfo;
begin
  fi := TFontInfo.Create(nil);
  fi.Assign(rvs.TextStyles[StyleNo]);
  case CommandChar of
    'b':
      if fsBold in fi.Style then
        fi.Style := fi.Style-[fsBold]
      else
        fi.Style := fi.Style+[fsBold];
    'i':
      if fsItalic in fi.Style then
        fi.Style := fi.Style-[fsItalic]
      else
        fi.Style := fi.Style+[fsItalic];
    'u':
      if fsUnderline in fi.Style then
        fi.Style := fi.Style-[fsUnderline]
      else
        fi.Style := fi.Style+[fsUnderline];
    'c':
      fi.Color := Color;
    'k':
      fi.BackColor := Color;
  end;
  Result := rvs.TextStyles.FindSuchStyle(StyleNo, fi, RVAllFontInfoProperties);
  if Result<0 then begin
    rvs.TextStyles.Add;
    Result := rvs.TextStyles.Count-1;
    rvs.TextStyles[Result].Assign(fi);
    rvs.TextStyles[Result].Standard := False;
  end;
  fi.Free;
end;
{------------------------------------------------------------------------------}
// Adding string with codes
procedure TForm1.ParseString(s: String);
var i: Integer;
    ParaNo, StyleNo, StartIndex: Integer;
    CodeStarted: Boolean;
    ColorChar: Char;
    ColorValue: String;
    {..................................}
    procedure AddString(StartIndex, EndIndex: Integer);
    var str: String;
        i: Integer;
        BSDeleted: Boolean;
    begin
      if EndIndex>=StartIndex then begin
        str := Copy(s, StartIndex, EndIndex-StartIndex+1);
        BSDeleted := False;
        for i := Length(str) downto 1 do     // replacing '\\' with '\'
          if (str[i]='\') and (i>1) and (str[i-1]='\') and not BSDeleted then begin
            Delete(str, i, 1);
            BSDeleted := True;
            end
          else
            BSDeleted := False;
        rv.AddNL(str, StyleNo, ParaNo);
        ParaNo := -1;
      end;
    end;
    {..................................}
begin
  ParaNo := 0;
  CodeStarted := False;
  StyleNo := 0;
  StartIndex := 1;
  ColorChar := #0;
  for i := 1 to Length(s) do begin
    if (ColorChar<>#0) then begin
      if (s[i] in ['0'..'9', 'a'..'f', 'A'..'F']) then begin
        ColorValue := ColorValue + s[i];
        if Length(ColorValue)=6 then begin
          AddString(StartIndex, i-2-6);
          StartIndex := i+1;
          StyleNo := GetStyleNo(ColorChar, GetStrColor(ColorValue), StyleNo);
          ColorChar := #0;
        end;
        continue;
      end;
      ColorChar := #0;
    end;
    case s[i] of
      '\':
        CodeStarted := not CodeStarted;
      'b', 'u', 'i', 'B', 'U', 'I':
        if CodeStarted then begin
          AddString(StartIndex, i-2);
          StartIndex := i+1;
          StyleNo := GetStyleNo(LowerCase(s[i])[1], clNone, StyleNo);
          CodeStarted := False;
        end;
      'c', 'C', 'k', 'K':
        if CodeStarted then begin
          ColorChar := LowerCase(s[i])[1];
          ColorValue := '';
          CodeStarted := False;
        end;
    end;
  end;
  AddString(StartIndex, Length(s));
end;


end.
